<?php

namespace Litecms\Recommendation\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Recommendation\Http\Requests\RecommendationRequest;
use Litecms\Recommendation\Interfaces\RecommendationRepositoryInterface;
use Litecms\Recommendation\Models\Recommendation;

/**
 * Resource controller class for recommendation.
 */
class RecommendationResourceController extends BaseController
{

    /**
     * Initialize recommendation resource controller.
     *
     * @param type RecommendationRepositoryInterface $recommendation
     *
     * @return null
     */
    public function __construct(RecommendationRepositoryInterface $recommendation)
    {
        parent::__construct();
        $this->repository = $recommendation;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Recommendation\Repositories\Criteria\RecommendationResourceCriteria::class);
    }

    /**
     * Display a list of recommendation.
     *
     * @return Response
     */
    public function index(RecommendationRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Recommendation\Repositories\Presenter\RecommendationPresenter::class)
                ->$function();
        }

        $recommendations = $this->repository->paginate();

        return $this->response->title(trans('recommendation::recommendation.names'))
            ->view('recommendation::recommendation.index', true)
            ->data(compact('recommendations', 'view'))
            ->output();
    }

    /**
     * Display recommendation.
     *
     * @param Request $request
     * @param Model   $recommendation
     *
     * @return Response
     */
    public function show(RecommendationRequest $request, Recommendation $recommendation)
    {

        if ($recommendation->exists) {
            $view = 'recommendation::recommendation.show';
        } else {
            $view = 'recommendation::recommendation.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('recommendation::recommendation.name'))
            ->data(compact('recommendation'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new recommendation.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(RecommendationRequest $request)
    {

        $recommendation = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('recommendation::recommendation.name')) 
            ->view('recommendation::recommendation.create', true) 
            ->data(compact('recommendation'))
            ->output();
    }

    /**
     * Create new recommendation.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(RecommendationRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $recommendation                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('recommendation::recommendation.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('recommendation/recommendation/' . $recommendation->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/recommendation/recommendation'))
                ->redirect();
        }

    }

    /**
     * Show recommendation for editing.
     *
     * @param Request $request
     * @param Model   $recommendation
     *
     * @return Response
     */
    public function edit(RecommendationRequest $request, Recommendation $recommendation)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('recommendation::recommendation.name'))
            ->view('recommendation::recommendation.edit', true)
            ->data(compact('recommendation'))
            ->output();
    }

    /**
     * Update the recommendation.
     *
     * @param Request $request
     * @param Model   $recommendation
     *
     * @return Response
     */
    public function update(RecommendationRequest $request, Recommendation $recommendation)
    {
        try {
            $attributes = $request->all();

            $recommendation->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('recommendation::recommendation.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('recommendation/recommendation/' . $recommendation->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('recommendation/recommendation/' . $recommendation->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the recommendation.
     *
     * @param Model   $recommendation
     *
     * @return Response
     */
    public function destroy(RecommendationRequest $request, Recommendation $recommendation)
    {
        try {

            $recommendation->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('recommendation::recommendation.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('recommendation/recommendation/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('recommendation/recommendation/' . $recommendation->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple recommendation.
     *
     * @param Model   $recommendation
     *
     * @return Response
     */
    public function delete(RecommendationRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('recommendation::recommendation.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('recommendation/recommendation'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/recommendation/recommendation'))
                ->redirect();
        }

    }

    /**
     * Restore deleted recommendations.
     *
     * @param Model   $recommendation
     *
     * @return Response
     */
    public function restore(RecommendationRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('recommendation::recommendation.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/recommendation/recommendation'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/recommendation/recommendation/'))
                ->redirect();
        }

    }

}
